@@ -28,21 +28,20 @@ Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 #include "gvdocumentimpl.h"
 
 class GVDocument;
+class QFile;
 
 class GVDocumentLoadedImpl : public GVDocumentImpl {
 Q_OBJECT
 public:
 	GVDocumentLoadedImpl(GVDocument* document);
+	void init();
 	~GVDocumentLoadedImpl();
 	
-	void modify(GVImageUtils::Orientation);
-	bool save(const KURL&, const char* format) const;
+	void transform(GVImageUtils::Orientation);
+	QString save(const KURL&, const QCString& format) const;
 
 protected:
-	virtual bool localSave(const QString& file, const char* format) const;
-	
-protected slots:
-	virtual void finishLoading();
+	virtual QString localSave(QFile* file, const QCString& format) const;
 };
 
 #endif /* GVDOCUMENTLOADEDIMPL_H */
