@@ -20,6 +20,7 @@ Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */
 
 // Qt
+#include <qframe.h>
 #include <qlayout.h>
 #include <qpainter.h>
 #include <qpen.h>
@@ -27,6 +28,7 @@ Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 #include <qpixmapcache.h>
 #include <qpushbutton.h>
 #include <qtimer.h>
+#include <qvaluevector.h>
 
 // KDE
 #include <kapplication.h>
@@ -35,6 +37,7 @@ Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 #include <kglobalsettings.h>
 #include <kiconloader.h>
 #include <klocale.h>
+#include <kprogress.h>
 #include <kstandarddirs.h>
 #include <kurldrag.h>
 #include <kwordwrap.h>
@@ -45,6 +48,13 @@ Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 #include "thumbnailloadjob.h"
 #include "gvbusylevelmanager.h"
 
+#define ENABLE_LOG
+#ifdef ENABLE_LOG
+#define LOG(x) kdDebug() << k_funcinfo << x << endl
+#else
+#define LOG(x) ;
+#endif
+
 #include "gvfilethumbnailview.moc"
 
 static const char* CONFIG_THUMBNAIL_SIZE="thumbnail size";
@@ -52,9 +62,82 @@ static const char* CONFIG_MARGIN_SIZE="margin size";
 static const char* CONFIG_WORD_WRAP_FILENAME="word wrap filename";
 
 
+class ProgressWidget : public QFrame {
+	KProgress* mProgressBar;
+	QPushButton* mStop;
+public:
+	ProgressWidget(GVFileThumbnailView* view, int count)
+	: QFrame(view)
+	{
+		QHBoxLayout* layout=new QHBoxLayout(this, 3, 3);
+		layout->setAutoAdd(true);
+		setFrameStyle( QFrame::StyledPanel | QFrame::Raised );
+		
+		mStop=new QPushButton(this);
+		mStop->setPixmap(SmallIcon("stop"));
+		mStop->setFlat(true);
+
+		mProgressBar=new KProgress(count, this);
+		mProgressBar->setFormat("%v/%m");
+
+		view->clipper()->installEventFilter(this);
+	}
+	
+	void polish() {
+		QFrame::polish();
+		setMinimumWidth(layout()->minimumSize().width());
+		//setFixedHeight( mProgressBar->height() );
+		setFixedHeight( mStop->height() );
+	}
+
+	void showEvent(QShowEvent*) {
+		updatePosition();
+	}
+
+	bool eventFilter(QObject*, QEvent* event) {
+		if (event->type()==QEvent::Resize) {
+			updatePosition();
+		}
+		return false;
+	}
+
+	void updatePosition() {
+		GVFileThumbnailView* view=static_cast<GVFileThumbnailView*>(parent());
+		QSize tmp=view->clipper()->size() - size();
+		move(tmp.width() - 2, tmp.height() - 2);
+	}
+
+	KProgress* progressBar() const { return mProgressBar; }
+	QPushButton* stopButton() const { return mStop; }
+};
+
+
+struct GVFileThumbnailView::Private {
+	ThumbnailSize mThumbnailSize;
+	int mMarginSize;
+	bool mUpdateThumbnailsOnNextShow;
+	QPixmap mWaitPixmap;
+	ProgressWidget* mProgressWidget;
+
+	QGuardedPtr<ThumbnailLoadJob> mThumbnailLoadJob;
+};
+
+
+static GVFileThumbnailViewItem* viewItem(const GVFileThumbnailView* view, const KFileItem* fileItem) {
+	if (!fileItem) return 0L;
+	return static_cast<GVFileThumbnailViewItem*>( const_cast<void*>(fileItem->extraData(view) ) );
+}
+
+
 GVFileThumbnailView::GVFileThumbnailView(QWidget* parent)
-: KIconView(parent), GVFileViewBase(), mUpdateThumbnailsOnNextShow(false), mThumbnailLoadJob(0L)
+: KIconView(parent), GVFileViewBase()
 {
+	d=new Private;
+	d->mUpdateThumbnailsOnNextShow=false;
+	d->mThumbnailLoadJob=0L;
+	d->mWaitPixmap=QPixmap(::locate("appdata", "thumbnail/wait.png"));
+	d->mProgressWidget=0L;
+
 	setAutoArrange(true);
 	QIconView::setSorting(true);
 	setItemsMovable(false);
@@ -63,8 +146,6 @@ GVFileThumbnailView::GVFileThumbnailView(QWidget* parent)
 	setSpacing(0);
 	setAcceptDrops(true);
 
-	mWaitPixmap=QPixmap(::locate("appdata", "thumbnail/wait.png"));
-
 	// We can't use KIconView::Execute mode because in this mode the current
 	// item is unselected after being clicked, so we use KIconView::Select mode
 	// and emit the execute() signal with slotClicked() ourself.
@@ -78,6 +159,8 @@ GVFileThumbnailView::GVFileThumbnailView(QWidget* parent)
 		this, SLOT(slotDropped(QDropEvent*)) );
 	connect(this, SIGNAL( contentsMoving( int, int )),
 		this, SLOT( slotContentsMoving( int, int )));
+	connect(this, SIGNAL(currentChanged(QIconViewItem*)),
+		this, SLOT(slotCurrentChanged(QIconViewItem*)) );
 
 	QIconView::setSelectionMode(Extended);
 
@@ -88,28 +171,39 @@ GVFileThumbnailView::GVFileThumbnailView(QWidget* parent)
 
 GVFileThumbnailView::~GVFileThumbnailView() {
 	stopThumbnailUpdate();
+	delete d;
 }
 
 
 void GVFileThumbnailView::setThumbnailSize(ThumbnailSize value) {
-	if (value==mThumbnailSize) return;
-	mThumbnailSize=value;
+	if (value==d->mThumbnailSize) return;
+	d->mThumbnailSize=value;
 	updateGrid();
 }
 
 
+ThumbnailSize GVFileThumbnailView::thumbnailSize() const {
+	return d->mThumbnailSize;
+}
+
+
 void GVFileThumbnailView::setMarginSize(int value) {
-	if (value==mMarginSize) return;
-	mMarginSize=value;
+	if (value==d->mMarginSize) return;
+	d->mMarginSize=value;
 	updateGrid();
 }
 
 
+int GVFileThumbnailView::marginSize() const {
+	return d->mMarginSize;
+}
+
+
 void GVFileThumbnailView::setThumbnailPixmap(const KFileItem* fileItem, const QPixmap& thumbnail, const QSize& size) {
-	GVFileThumbnailViewItem* iconItem=viewItem(fileItem);
+	GVFileThumbnailViewItem* iconItem=viewItem(this, fileItem);
 	if (!iconItem) return;
 
-	int pixelSize=mThumbnailSize.pixelSize();
+	int pixelSize=d->mThumbnailSize.pixelSize();
 
 	// Draw the thumbnail to the center of the icon
 	QPainter painter(iconItem->pixmap());
@@ -127,8 +221,9 @@ void GVFileThumbnailView::setThumbnailPixmap(const KFileItem* fileItem, const QP
 	
 	iconItem->repaint();
 
-	// Notify others that one thumbnail has been updated
-	emit updatedOneThumbnail();
+	// Notify progress
+	Q_ASSERT(d->mProgressWidget);
+	d->mProgressWidget->progressBar()->advance(1);
 }
 
 
@@ -136,8 +231,8 @@ void GVFileThumbnailView::setThumbnailPixmap(const KFileItem* fileItem, const QP
 
 void GVFileThumbnailView::setShownFileItem(KFileItem* fileItem) {
 	if( fileItem == mShownFileItem ) return;
-	GVFileThumbnailViewItem* oldShownItem=viewItem(mShownFileItem);
-	GVFileThumbnailViewItem* newShownItem=viewItem(fileItem);
+	GVFileThumbnailViewItem* oldShownItem=viewItem(this, mShownFileItem);
+	GVFileThumbnailViewItem* newShownItem=viewItem(this, fileItem);
 
 	GVFileViewBase::setShownFileItem(fileItem);
 	if (oldShownItem) repaintItem(oldShownItem);
@@ -153,66 +248,86 @@ void GVFileThumbnailView::setShownFileItem(KFileItem* fileItem) {
 void GVFileThumbnailView::startThumbnailUpdate() {
 	// Delay thumbnail update if the widget is not visible
 	if (!isVisible()) {
-		mUpdateThumbnailsOnNextShow=true;
+		d->mUpdateThumbnailsOnNextShow=true;
 		return;
 	}
-	mUpdateThumbnailsOnNextShow=false;
+	d->mUpdateThumbnailsOnNextShow=false;
 	stopThumbnailUpdate(); // just in case
 	doStartThumbnailUpdate(items());
 }
 
 
 void GVFileThumbnailView::doStartThumbnailUpdate(const KFileItemList* list) {
-	GVBusyLevelManager::instance()->setBusyLevel( this, BUSY_THUMBNAILS );
-	mThumbnailLoadJob = new ThumbnailLoadJob(list, mThumbnailSize);
+	QValueVector<const KFileItem*> imageList;
+	imageList.reserve( list->count());
+	QPtrListIterator<KFileItem> it(*list);
+	for (;it.current(); ++it) {
+		KFileItem* item=it.current();
+		if (!item->isDir() && !GVArchive::fileItemIsArchive(item)) {
+			imageList.append( item );
+		}
+	}
+	if (imageList.empty()) return;
 
-	connect(mThumbnailLoadJob, SIGNAL(thumbnailLoaded(const KFileItem*, const QPixmap&, const QSize&)),
+	GVBusyLevelManager::instance()->setBusyLevel( this, BUSY_THUMBNAILS );
+	
+	d->mThumbnailLoadJob = new ThumbnailLoadJob(&imageList, d->mThumbnailSize);
+
+	connect(d->mThumbnailLoadJob, SIGNAL(thumbnailLoaded(const KFileItem*, const QPixmap&, const QSize&)),
 		this, SLOT(setThumbnailPixmap(const KFileItem*,const QPixmap&, const QSize&)) );
-	connect(mThumbnailLoadJob, SIGNAL(result(KIO::Job*)),
-		this, SIGNAL(updateEnded()) );
-	connect(this, SIGNAL(updateEnded()),
+	connect(d->mThumbnailLoadJob, SIGNAL(result(KIO::Job*)),
 		this, SLOT(slotUpdateEnded()) );
 
-	emit updateStarted(list->count());
+	Q_ASSERT(!d->mProgressWidget);
+	d->mProgressWidget=new ProgressWidget(this, imageList.count() );
+	connect(d->mProgressWidget->stopButton(), SIGNAL(clicked()),
+		this, SLOT(stopThumbnailUpdate()) );
+	d->mProgressWidget->show();
 	slotBusyLevelChanged( GVBusyLevelManager::instance()->busyLevel());
 	// start updating at visible position
 	slotContentsMoving( contentsX(), contentsY());
-	mThumbnailLoadJob->start();
+	d->mThumbnailLoadJob->start();
 }
 
 
 void GVFileThumbnailView::stopThumbnailUpdate() {
-	if (!mThumbnailLoadJob.isNull()) {
-		emit updateEnded();
-		mThumbnailLoadJob->kill();
+	if (!d->mThumbnailLoadJob.isNull()) {
+		d->mThumbnailLoadJob->kill(false);
 	}
 }
 
 
 void GVFileThumbnailView::slotUpdateEnded() {
+	Q_ASSERT(d->mProgressWidget);
+	delete d->mProgressWidget;
+	d->mProgressWidget=0L;
 	GVBusyLevelManager::instance()->setBusyLevel( this, BUSY_NONE );
 }
 
+
 void GVFileThumbnailView::updateThumbnail(const KFileItem* fileItem) {
+	if (fileItem->isDir() || GVArchive::fileItemIsArchive(fileItem)) {
+		return;
+	}
 
 	ThumbnailLoadJob::deleteImageThumbnail(fileItem->url());
-	if (mThumbnailLoadJob.isNull()) {
+	if (d->mThumbnailLoadJob.isNull()) {
 		KFileItemList list;
 		list.append(fileItem);
 		doStartThumbnailUpdate(&list);
 	} else {
-		mThumbnailLoadJob->appendItem(fileItem);
+		d->mThumbnailLoadJob->appendItem(fileItem);
 	}
 }
 
 // temporarily stop loading thumbnails when busy loading the selected image,
 // otherwise thumbnail loading slows it down
 void GVFileThumbnailView::slotBusyLevelChanged(GVBusyLevel level) {
-	if( !mThumbnailLoadJob.isNull()) {
+	if( !d->mThumbnailLoadJob.isNull()) {
 		if( level > BUSY_THUMBNAILS ) {
-			mThumbnailLoadJob->suspend();
+			d->mThumbnailLoadJob->suspend();
 		} else {
-			mThumbnailLoadJob->resume();
+			d->mThumbnailLoadJob->resume();
 		}
 	}
 }
@@ -234,7 +349,7 @@ void GVFileThumbnailView::insertItem(KFileItem* item) {
 
 	bool isDirOrArchive=item->isDir() || GVArchive::fileItemIsArchive(item);
 
-	int pixelSize=mThumbnailSize.pixelSize();
+	int pixelSize=d->mThumbnailSize.pixelSize();
 	QPixmap thumbnail(pixelSize,pixelSize);
 	QPainter painter(&thumbnail);
 	painter.fillRect(0,0,pixelSize,pixelSize,paletteBackgroundColor());
@@ -251,9 +366,9 @@ void GVFileThumbnailView::insertItem(KFileItem* item) {
 		painter.setPen(colorGroup().button());
 		painter.drawRect(0,0,pixelSize,pixelSize);
 		painter.drawPixmap(
-			(pixelSize-mWaitPixmap.width())/2,
-			(pixelSize-mWaitPixmap.height())/2,
-			mWaitPixmap);
+			(pixelSize-d->mWaitPixmap.width())/2,
+			(pixelSize-d->mWaitPixmap.height())/2,
+			d->mWaitPixmap);
 	}
 
 	// Create icon item
@@ -268,7 +383,7 @@ void GVFileThumbnailView::insertItem(KFileItem* item) {
 void GVFileThumbnailView::updateView(const KFileItem* fileItem) {
 	if (!fileItem) return;
 
-	GVFileThumbnailViewItem* iconItem=viewItem(fileItem);
+	GVFileThumbnailViewItem* iconItem=viewItem(this, fileItem);
 	if (iconItem) {
 		iconItem->setText(fileItem->text());
 		updateThumbnail(fileItem);
@@ -280,7 +395,7 @@ void GVFileThumbnailView::updateView(const KFileItem* fileItem) {
 void GVFileThumbnailView::ensureItemVisible(const KFileItem* fileItem) {
 	if (!fileItem) return;
 
-	GVFileThumbnailViewItem* iconItem=viewItem(fileItem);
+	GVFileThumbnailViewItem* iconItem=viewItem(this, fileItem);
 	if (iconItem) QIconView::ensureItemVisible(iconItem);
 }
 
@@ -288,7 +403,7 @@ void GVFileThumbnailView::ensureItemVisible(const KFileItem* fileItem) {
 void GVFileThumbnailView::setCurrentItem(const KFileItem* fileItem) {
 	if (!fileItem) return;
 
-	GVFileThumbnailViewItem* iconItem=viewItem(fileItem);
+	GVFileThumbnailViewItem* iconItem=viewItem(this, fileItem);
 	if (iconItem) QIconView::setCurrentItem(iconItem);
 }
 
@@ -296,7 +411,7 @@ void GVFileThumbnailView::setCurrentItem(const KFileItem* fileItem) {
 void GVFileThumbnailView::setSelected(const KFileItem* fileItem,bool enable) {
 	if (!fileItem) return;
 
-	GVFileThumbnailViewItem* iconItem=viewItem(fileItem);
+	GVFileThumbnailViewItem* iconItem=viewItem(this, fileItem);
 	if (iconItem) QIconView::setSelected(iconItem,enable);
 }
 
@@ -304,7 +419,7 @@ void GVFileThumbnailView::setSelected(const KFileItem* fileItem,bool enable) {
 bool GVFileThumbnailView::isSelected(const KFileItem* fileItem) const {
 	if (!fileItem) return false;
 
-	GVFileThumbnailViewItem* iconItem=viewItem(fileItem);
+	GVFileThumbnailViewItem* iconItem=viewItem(this, fileItem);
 	if (!iconItem) return false;
 
 	return iconItem->isSelected();
@@ -315,13 +430,13 @@ void GVFileThumbnailView::removeItem(const KFileItem* fileItem) {
 	if (!fileItem) return;
 
 	// Remove it from the image preview job
-	if (!mThumbnailLoadJob.isNull())
-		mThumbnailLoadJob->itemRemoved(fileItem);
+	if (!d->mThumbnailLoadJob.isNull())
+		d->mThumbnailLoadJob->itemRemoved(fileItem);
 
 	if (fileItem==mShownFileItem) mShownFileItem=0L;
 
 	// Remove it from our view
-	GVFileThumbnailViewItem* iconItem=viewItem(fileItem);
+	GVFileThumbnailViewItem* iconItem=viewItem(this, fileItem);
 	if (iconItem) delete iconItem;
 	KFileView::removeItem(fileItem);
 	arrangeItemsInGrid();
@@ -336,7 +451,7 @@ KFileItem* GVFileThumbnailView::firstFileItem() const {
 
 
 KFileItem* GVFileThumbnailView::prevItem(const KFileItem* fileItem) const {
-	const GVFileThumbnailViewItem* iconItem=viewItem(fileItem);
+	const GVFileThumbnailViewItem* iconItem=viewItem(this, fileItem);
 	if (!iconItem) return 0L;
 
 	iconItem=static_cast<const GVFileThumbnailViewItem*>(iconItem->prevItem());
@@ -355,7 +470,7 @@ KFileItem* GVFileThumbnailView::currentFileItem() const {
 
 
 KFileItem* GVFileThumbnailView::nextItem(const KFileItem* fileItem) const {
-	const GVFileThumbnailViewItem* iconItem=viewItem(fileItem);
+	const GVFileThumbnailViewItem* iconItem=viewItem(this, fileItem);
 	if (!iconItem) return 0L;
 
 	iconItem=static_cast<const GVFileThumbnailViewItem*>(iconItem->nextItem());
@@ -372,11 +487,11 @@ void GVFileThumbnailView::setSorting(QDir::SortSpec spec) {
 	KFileItemListIterator it( *items() );
 
 	for ( ; (item = it.current() ); ++it ) {
-		QIconViewItem* iconItem=viewItem(item);
+		QIconViewItem* iconItem=viewItem(this, item);
 		if (iconItem) setSortingKey(iconItem, item);
 	}
 
-	KIconView::sort();
+	KIconView::sort(! (spec & QDir::Reversed) );
 }
 
 //--------------------------------------------------------------------------
@@ -396,9 +511,9 @@ void GVFileThumbnailView::slotDropped(QDropEvent* event) {
 
 void GVFileThumbnailView::showEvent(QShowEvent* event) {
 	KIconView::showEvent(event);
-	if (!mUpdateThumbnailsOnNextShow) return;
+	if (!d->mUpdateThumbnailsOnNextShow) return;
 
-	mUpdateThumbnailsOnNextShow=false;
+	d->mUpdateThumbnailsOnNextShow=false;
 	QTimer::singleShot(0, this, SLOT(startThumbnailUpdate()));
 }
 
@@ -409,7 +524,7 @@ void GVFileThumbnailView::showEvent(QShowEvent* event) {
 //
 //--------------------------------------------------------------------------
 void GVFileThumbnailView::updateGrid() {
-	setGridX(mThumbnailSize.pixelSize() + mMarginSize);
+	setGridX(d->mThumbnailSize.pixelSize() + d->mMarginSize);
 }
 
 
@@ -419,14 +534,17 @@ void GVFileThumbnailView::setSortingKey(QIconViewItem *iconItem, const KFileItem
 	QDir::SortSpec spec = KFileView::sorting();
 	bool isDirOrArchive=item->isDir() || GVArchive::fileItemIsArchive(item);
 
+	QString key;
 	if ( spec & QDir::Time )
-		iconItem->setKey( sortingKey( item->time( KIO::UDS_MODIFICATION_TIME ),
-								  isDirOrArchive, spec ));
+		key=sortingKey( item->time( KIO::UDS_MODIFICATION_TIME ),
+								  isDirOrArchive, spec );
 	else if ( spec & QDir::Size )
-		iconItem->setKey( sortingKey( item->size(), isDirOrArchive, spec ));
+		key=sortingKey( item->size(), isDirOrArchive, spec );
 
 	else // Name or Unsorted
-		iconItem->setKey( sortingKey( item->text(), isDirOrArchive, spec ));
+		key=sortingKey( item->text(), isDirOrArchive, spec );
+
+	iconItem->setKey(key);
 }
 
 
@@ -441,7 +559,6 @@ void GVFileThumbnailView::slotDoubleClicked(QIconViewItem* iconItem) {
 	GVFileThumbnailViewItem* thumbItem=static_cast<GVFileThumbnailViewItem*>(iconItem);
 
 	KFileItem* fileItem=thumbItem->fileItem();
-	if (!fileItem) return;
 
 	if (fileItem->isDir() || GVArchive::fileItemIsArchive(fileItem)) {
 		emit executed(iconItem);
@@ -455,30 +572,50 @@ void GVFileThumbnailView::slotClicked(QIconViewItem* iconItem) {
 	GVFileThumbnailViewItem* thumbItem=static_cast<GVFileThumbnailViewItem*>(iconItem);
 
 	KFileItem* fileItem=thumbItem->fileItem();
-	if (!fileItem) return;
 
 	if (fileItem->isDir() || GVArchive::fileItemIsArchive(fileItem)) {
 		emit executed(iconItem);
 	}
 }
 
-/**
- * when generating thumbnails, make the first visible (not loaded yet)
- * thumbnail be the next one processed by the thumbnail job
- */
 void GVFileThumbnailView::slotContentsMoving( int x, int y ) {
-	if (mThumbnailLoadJob.isNull()) return;
-	QRect r( x, y, visibleWidth(), visibleHeight());
-	GVFileThumbnailViewItem* pos = static_cast< GVFileThumbnailViewItem* >( findFirstVisibleItem( r ));
-	GVFileThumbnailViewItem* last = static_cast< GVFileThumbnailViewItem* >( findLastVisibleItem( r ));
-	while( pos != NULL ) {
-		KFileItem* fileItem = pos->fileItem();
-		if( fileItem ) {
-			if( mThumbnailLoadJob->setNextItem(fileItem)) return;
-		}
-		if( pos == last ) break;
-		pos = static_cast< GVFileThumbnailViewItem* >( pos->nextItem());
+	updateVisibilityInfo( x, y ); // use x,y, the signal is emitted before moving
+}
+
+void GVFileThumbnailView::slotCurrentChanged(QIconViewItem*) {
+	// trigger generating thumbnails from the current one
+	updateVisibilityInfo( contentsX(), contentsY());
+}
+
+/**
+ * when generating thumbnails, make the current thumbnail
+ * to be the next one processed by the thumbnail job, if visible,
+ * otherwise use the first visible thumbnail
+ */
+void GVFileThumbnailView::updateVisibilityInfo( int x, int y ) {
+	if (d->mThumbnailLoadJob.isNull()) return;
+	
+	QRect rect( x, y, visibleWidth(), visibleHeight());
+	GVFileThumbnailViewItem* first = static_cast< GVFileThumbnailViewItem* >( findFirstVisibleItem( rect ));
+	if (!first) {
+		d->mThumbnailLoadJob->setPriorityItems(NULL,NULL,NULL);
+		return;
 	}
+	
+	GVFileThumbnailViewItem* last = static_cast< GVFileThumbnailViewItem* >( findLastVisibleItem( rect ));
+	Q_ASSERT(last); // If we get a first item, then there must be a last
+	
+	if (currentItem() && currentItem()->intersects(rect)) {
+		KFileItem* fileItem = currentFileItem();
+		d->mThumbnailLoadJob->setPriorityItems(fileItem,
+			first->fileItem(), last->fileItem());
+		return;
+	}
+	
+	d->mThumbnailLoadJob->setPriorityItems(
+		first->fileItem(),
+		first->fileItem(),
+		last->fileItem());
 }
 
 //--------------------------------------------------------------------------
@@ -520,8 +657,8 @@ void GVFileThumbnailView::startDrag() {
 void GVFileThumbnailView::readConfig(KConfig* config,const QString& group) {
 	config->setGroup(group);
 
-	mThumbnailSize=config->readEntry(CONFIG_THUMBNAIL_SIZE);
-	mMarginSize=config->readNumEntry(CONFIG_MARGIN_SIZE,5);
+	d->mThumbnailSize=config->readEntry(CONFIG_THUMBNAIL_SIZE);
+	d->mMarginSize=config->readNumEntry(CONFIG_MARGIN_SIZE,5);
 
 	updateGrid();
 	setWordWrapIconText(config->readBoolEntry(CONFIG_WORD_WRAP_FILENAME,false));
@@ -529,8 +666,8 @@ void GVFileThumbnailView::readConfig(KConfig* config,const QString& group) {
 }
 
 void GVFileThumbnailView::kpartConfig() {
-	mThumbnailSize=ThumbnailSize::MED;
-	mMarginSize=5;
+	d->mThumbnailSize=ThumbnailSize::MED;
+	d->mMarginSize=5;
 
 	updateGrid();
 	setWordWrapIconText(false);
@@ -540,8 +677,8 @@ void GVFileThumbnailView::kpartConfig() {
 
 void GVFileThumbnailView::writeConfig(KConfig* config,const QString& group) const {
 	config->setGroup(group);
-	config->writeEntry(CONFIG_THUMBNAIL_SIZE,QString(mThumbnailSize));
-	config->writeEntry(CONFIG_MARGIN_SIZE,mMarginSize);
+	config->writeEntry(CONFIG_THUMBNAIL_SIZE,QString(d->mThumbnailSize));
+	config->writeEntry(CONFIG_MARGIN_SIZE,d->mMarginSize);
 	config->writeEntry(CONFIG_WORD_WRAP_FILENAME,wordWrapIconText());
 }
 
