@@ -1,7 +1,7 @@
 /*
  * qxcfi.cpp: A Qt 3 plug-in for reading GIMP XCF image files
  * Copyright (C) 2001 lignum Computing, Inc. <allen@lignumcomputing.com>
- * $Id: qxcfi.cpp,v 1.6 2004/05/30 15:29:50 gateau Exp $
+ * $Id: qxcfi.cpp,v 1.9 2004/11/22 23:21:50 gateau Exp $
  *
  * This plug-in is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -20,6 +20,7 @@
  */
 
 #include <qiodevice.h>
+#include <kdeversion.h>
 #include <stdlib.h>
 #include "qxcfi.h"
 
@@ -784,20 +785,20 @@ bool XCFImageFormat::loadImageProperties ( SafeDataStream& xcf_io,
 
     case PROP_PARASITES:
       while ( !property.atEnd() ) {
-	char* tag;
-	Q_UINT32 size;
+ char* tag;
+ Q_UINT32 size;
 
-	property.readBytes( tag, size );
+ property.readBytes( tag, size );
 
-	Q_UINT32 flags;
-	char* data;
-	property >> flags >> data;
+ Q_UINT32 flags;
+ char* data;
+ property >> flags >> data;
 
-	if ( strcmp( tag, "gimp-comment" ) == 0 )
-	  xcf_image.image.setText( "Comment", 0, data );
+ if ( strcmp( tag, "gimp-comment" ) == 0 )
+   xcf_image.image.setText( "Comment", 0, data );
 
-	delete[] tag;
-	delete[] data;
+ delete[] tag;
+ delete[] data;
       }
       break;
 
@@ -2372,10 +2373,14 @@ void XCFImageFormat::dissolveAlphaPixels ( QImage& image, int x, int y )
       uchar alpha = image.pixelIndex( k, l );
 
       if ( rand_val > alpha ) {
-	image.setPixel( k, l, 0 );
+ image.setPixel( k, l, 0 );
       }
     }
   }
 }
 
+#if KDE_IS_VERSION(3, 3, 89)
+KDE_Q_EXPORT_PLUGIN( XCFImageFormat )
+#else
 Q_EXPORT_PLUGIN( XCFImageFormat )
+#endif
