@@ -1,26 +1,26 @@
 // vim: set tabstop=4 shiftwidth=4 noexpandtab
-/*  Gwenview - A simple image viewer for KDE
-    Copyright 2000-2004 Aurlien Gteau
-    This class is based on the ImagePreviewJob class from Konqueror.
-    Original copyright follows.
+/*	Gwenview - A simple image viewer for KDE
+	Copyright 2000-2004 Aurlien Gteau
+	This class is based on the ImagePreviewJob class from Konqueror.
+	Original copyright follows.
 */
-/*  This file is part of the KDE project
-    Copyright (C) 2000 David Faure <faure@kde.org>
-                  2000 Carsten Pfeiffer <pfeiffer@kde.org>
+/*	This file is part of the KDE project
+	Copyright (C) 2000 David Faure <faure@kde.org>
+				  2000 Carsten Pfeiffer <pfeiffer@kde.org>
 
-    This program is free software; you can redistribute it and/or modify
-    it under the terms of the GNU General Public License as published by
-    the Free Software Foundation; either version 2 of the License, or
-    (at your option) any later version.
+	This program is free software; you can redistribute it and/or modify
+	it under the terms of the GNU General Public License as published by
+	the Free Software Foundation; either version 2 of the License, or
+	(at your option) any later version.
 
-    This program is distributed in the hope that it will be useful,
-    but WITHOUT ANY WARRANTY; without even the implied warranty of
-    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-    GNU General Public License for more details.
+	This program is distributed in the hope that it will be useful,
+	but WITHOUT ANY WARRANTY; without even the implied warranty of
+	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+	GNU General Public License for more details.
 
-    You should have received a copy of the GNU General Public License
-    along with this program; if not, write to the Free Software
-    Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
+	You should have received a copy of the GNU General Public License
+	along with this program; if not, write to the Free Software
+	Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */
 
 #include <sys/types.h>
@@ -54,8 +54,8 @@ extern "C" {
 }
 
 // Local
-#include "gvarchive.h"
-#include "gvimageutils.h"
+#include "gvimageutils/jpegcontent.h"
+#include "gvimageutils/gvimageutils.h"
 #include "thumbnailloadjob.moc"
 
 //#define ENABLE_LOG
@@ -68,7 +68,11 @@ extern "C" {
 
 
 
-QString generateOriginalURI(KURL url) {
+const char CONFIG_STORE_THUMBNAILS_IN_CACHE[]="path";
+
+static bool sStoreThumbnailsInCache;
+
+static QString generateOriginalURI(KURL url) {
 	// Don't include the password if any
 	url.setPass(QString::null);
 	
@@ -82,7 +86,7 @@ QString generateOriginalURI(KURL url) {
 }
 
 
-QString generateThumbnailPath(const QString& uri) {
+static QString generateThumbnailPath(const QString& uri) {
 	KMD5 md5( QFile::encodeName(uri) );
 	QString baseDir=ThumbnailLoadJob::thumbnailBaseDir();
 	return baseDir + QString(QFile::encodeName( md5.hexDigest())) + ".png";
@@ -96,7 +100,8 @@ QString generateThumbnailPath(const QString& uri) {
 void ThumbnailThread::load(
 	const QString& originalURI, time_t originalTime, int originalSize, const QString& originalMimeType,
 	const QString& pixPath,
-	const QString& thumbnailPath)
+	const QString& thumbnailPath,
+	ThumbnailSize size, bool storeThumbnail)
 {
 	QMutexLocker lock( &mMutex );
 	assert( mPixPath.isNull());
@@ -107,6 +112,8 @@ void ThumbnailThread::load(
 	mOriginalMimeType = TSDeepCopy(originalMimeType);
 	mPixPath = TSDeepCopy(pixPath);
 	mThumbnailPath = TSDeepCopy(thumbnailPath);
+	mThumbnailSize = size;
+	mStoreThumbnailsInCache = storeThumbnail;
 	if(!running()) start();
 	mCond.wakeOne();
 }
@@ -128,15 +135,28 @@ void ThumbnailThread::run() {
 void ThumbnailThread::loadThumbnail() {
 	mImage = QImage();
 	bool loaded=false;
-	int width, height;
+	int width = -1, height;
 	
 	// If it's a JPEG, try to load a small image directly from the file
 	if (isJPEG(mPixPath)) {
-		loaded=loadJPEG(mPixPath, mImage, width, height);
+		//GVImageUtils::Orientation orientation = GVImageUtils::NOT_AVAILABLE;
+		//GVImageUtils::getOrientationAndThumbnail(mPixPath,orientation, mImage);
+		GVImageUtils::JPEGContent content;
+		content.load(mPixPath);
+		GVImageUtils::Orientation orientation = content.orientation();
+		mImage = content.thumbnail();
+	
+		if( !mImage.isNull()
+			&& ( mImage.width() >= mThumbnailSize.pixelSize() // don't use small thumbnails
+			|| mImage.height() >= mThumbnailSize.pixelSize() )) {
+			loaded = true; // don't set width/height, so that no thumbnail is saved
+		}
+		if(!loaded) {
+			loaded=loadJPEG(mPixPath, mImage, width, height);
+		}
 		if (loaded) {
 			// Rotate if necessary
-			GVImageUtils::Orientation orientation=GVImageUtils::getOrientation(mPixPath);
-			mImage=GVImageUtils::modify(mImage,orientation);
+			mImage=GVImageUtils::transform(mImage,orientation);
 		}
 	}
 	// File is not a JPEG, or JPEG optimized load failed, load file using Qt
@@ -152,7 +172,7 @@ void ThumbnailThread::loadThumbnail() {
 			if (width<=thumbSize && height<=thumbSize) {
 				mImage=bigImg;
 			} else {
-				mImage=GVImageUtils::scale(bigImg,thumbSize,thumbSize,GVImageUtils::SMOOTH_NONE,QImage::ScaleMin);
+				mImage=GVImageUtils::scale(bigImg,thumbSize,thumbSize,GVImageUtils::SMOOTH_FAST,QImage::ScaleMin);
 			}
 			loaded = true;
 		}
@@ -160,7 +180,7 @@ void ThumbnailThread::loadThumbnail() {
 
 	if( testCancel()) return;
 
-	if( loaded ) {
+	if( mStoreThumbnailsInCache && loaded && width != -1 ) {
 		mImage.setText("Thumb::URI", 0, mOriginalURI);
 		mImage.setText("Thumb::MTime", 0, QString::number(mOriginalTime));
 		mImage.setText("Thumb::Size", 0, QString::number(mOriginalSize));
@@ -276,7 +296,7 @@ bool ThumbnailThread::loadJPEG( const QString &pixPath, QImage& image, int& widt
 	int newx = size*cinfo.output_width / newMax;
 	int newy = size*cinfo.output_height / newMax;
 
-	image=GVImageUtils::scale(image,newx, newy,GVImageUtils::SMOOTH_NONE);
+	image=GVImageUtils::scale(image,newx, newy,GVImageUtils::SMOOTH_FAST);
 
 	jpeg_destroy_decompress(&cinfo);
 	fclose(inputFile);
@@ -293,7 +313,7 @@ bool ThumbnailThread::loadJPEG( const QString &pixPath, QImage& image, int& widt
 QString ThumbnailLoadJob::thumbnailBaseDir() {
 	static QString dir;
 	if (!dir.isEmpty()) return dir;
-    dir = QDir::homeDirPath() + "/.thumbnails/normal/";
+	dir = QDir::homeDirPath() + "/.thumbnails/normal/";
 	return dir;
 }
 
@@ -309,23 +329,42 @@ void ThumbnailLoadJob::deleteImageThumbnail(const KURL& url) {
 // ThumbnailLoadJob implementation
 //
 //------------------------------------------------------------------------
-ThumbnailLoadJob::ThumbnailLoadJob(const KFileItemList* items, ThumbnailSize size)
-: KIO::Job(false), mState( STATE_NEXTTHUMB ), mThumbnailSize(size), mSuspended( false )
+
+
+/*
+
+ This class tries to first generate the most important thumbnails, i.e.
+ first the currently selected one, then the ones that are visible, and then
+ the rest, the closer the the currently selected one the sooner
+
+ mAllItems contains all thumbnails
+ mItems contains pending thumbnails, in the priority order
+ mCurrentItem is currently processed thumbnail, already removed from mItems
+ mProcessedState needs to match mAllItems, and contains information about every
+   thumbnail whether it has been already processed
+
+ thumbnailIndex() returns index of a thumbnail in mAllItems, or -1
+ updateItemsOrder() builds mItems from mAllItems
+*/
+
+ThumbnailLoadJob::ThumbnailLoadJob(const QValueVector<const KFileItem*>* items, ThumbnailSize size)
+: KIO::Job(false), mState( STATE_NEXTTHUMB ),
+  mCurrentVisibleIndex( -1 ), mFirstVisibleIndex( -1 ), mLastVisibleIndex( -1 ),
+  mThumbnailSize(size), mSuspended( false )
 {
 	LOG("");
-    
+	
 	mBrokenPixmap=KGlobal::iconLoader()->loadIcon("file_broken", 
 		KIcon::NoGroup, ThumbnailSize(ThumbnailSize::SMALL).pixelSize());
 
 	// Look for images and store the items in our todo list
-	mItems=*items;
-
-	if (mItems.isEmpty()) return;
+	Q_ASSERT(!items->empty());
+	mAllItems=*items;
+	mProcessedState.resize( mAllItems.count());
+	qFill( mProcessedState.begin(), mProcessedState.end(), false );
+	mCurrentItem = NULL;
 
 	connect( &mThumbnailThread, SIGNAL( done( const QImage& )), SLOT( thumbnailReady( const QImage& )));
-	
-	// Move to the first item
-	mNextItem = mItems.first();
 }
 
 
@@ -337,6 +376,10 @@ ThumbnailLoadJob::~ThumbnailLoadJob() {
 
 
 void ThumbnailLoadJob::start() {
+	// build mItems from mAllItems if not done yet
+	if (mLastVisibleIndex == -1 ) {
+		setPriorityItems( NULL, NULL, NULL );
+	}
 	if (mItems.isEmpty()) {
 		LOG("Nothing to do");
 		emit result(this);
@@ -360,83 +403,133 @@ void ThumbnailLoadJob::resume() {
 
 //-Internal--------------------------------------------------------------
 void ThumbnailLoadJob::appendItem(const KFileItem* item) {
-	mItems.append(item);
+	int index = thumbnailIndex( item );
+	if( index >= 0 ) {
+		mProcessedState[ index ] = false;
+		return;
+	}
+	mAllItems.append(item);
+	mProcessedState.append( false );
+	updateItemsOrder();
 }
 
 
 void ThumbnailLoadJob::itemRemoved(const KFileItem* item) {
-	mItems.removeRef(item);
-	if( item == mNextItem ) mNextItem = mItems.current();
+	Q_ASSERT(item);
+
+	// If we are removing the next item, update to be the item after or the
+	// first if we removed the last item
+	mItems.remove( item );
+	int index = thumbnailIndex( item );
+	if( index >= 0 ) {
+		mAllItems.erase( mAllItems.begin() + index );
+		mProcessedState.erase( mProcessedState.begin() + index );
+	}
 
 	if (item == mCurrentItem) {
 		// Abort
-		subjobs.first()->kill();
-		subjobs.removeFirst();
+		mCurrentItem = NULL;
+		if (subjobs.first()) {
+			subjobs.first()->kill();
+			subjobs.removeFirst();
+		}
 		determineNextIcon();
 	}
 }
 
 
-bool ThumbnailLoadJob::setNextItem(const KFileItem* item) {
-	// Move mNextItem to the next item to be processed
-	if( mItems.containsRef( item )) {
-		mNextItem = item;
-		return true;
+void ThumbnailLoadJob::setPriorityItems(const KFileItem* current, const KFileItem* first, const KFileItem* last) {
+	if( mAllItems.isEmpty()) {
+		mCurrentVisibleIndex = mFirstVisibleIndex = mLastVisibleIndex = 0;
+		return;
 	}
-	mNextItem = mItems.first();
-	return false;
+	mFirstVisibleIndex = -1;
+	mLastVisibleIndex = - 1;
+	mCurrentVisibleIndex = -1;
+	if( first != NULL ) mFirstVisibleIndex = thumbnailIndex( first );
+	if( last != NULL ) mLastVisibleIndex = thumbnailIndex( last );
+	if( current != NULL ) mCurrentVisibleIndex = thumbnailIndex( current );
+	if( mFirstVisibleIndex == -1 ) mFirstVisibleIndex = 0;
+	if( mLastVisibleIndex == -1 ) mLastVisibleIndex = mAllItems.count() - 1;
+	if( mCurrentVisibleIndex == -1 ) mCurrentVisibleIndex = mFirstVisibleIndex;
+	updateItemsOrder();
 }
 
+void ThumbnailLoadJob::updateItemsOrder() {
+	mItems.clear();
+	int forward = mCurrentVisibleIndex + 1;
+	int backward = mCurrentVisibleIndex;
+	int first = mFirstVisibleIndex;
+	int last = mLastVisibleIndex;
+	updateItemsOrderHelper( forward, backward, first, last );
+	if( first != 0 || last != int( mAllItems.count()) - 1 ) {
+		// add non-visible items
+		updateItemsOrderHelper( last + 1, first - 1, 0, mAllItems.count() - 1);
+	}
+}
+
+void ThumbnailLoadJob::updateItemsOrderHelper( int forward, int backward, int first, int last ) {
+	// start from the current item, add one following it, and one preceding it, for all visible items
+	while( forward <= last || backward >= first ) {
+		// start with backward - that's the curent item for the first time
+		while( backward >= first ) {
+			if( !mProcessedState[ backward ] ) {
+				mItems.append( mAllItems[ backward ] );
+				--backward;
+				break;
+			}
+			--backward;
+		}
+		while( forward <= last ) {
+			if( !mProcessedState[ forward ] ) {
+				mItems.append( mAllItems[ forward ] );
+				++forward;
+				break;
+			}
+			++forward;
+		}
+	}
+}
 
 void ThumbnailLoadJob::determineNextIcon() {
 	mState = STATE_NEXTTHUMB;
 	if( mSuspended ) {
 		return;
 	}
-	if( mNextItem != mItems.current()) mItems.findRef( mNextItem );
-	assert( mNextItem == mItems.current());
-	// Skip non images 
-	while (true) {
-		KFileItem* item=mItems.current();
-		if (!item) break;
-		if (item->isDir() || GVArchive::fileItemIsArchive(item)) {
-			mItems.remove();
-			mNextItem = mItems.current();
-		} else {
-			break;
-		}
-	}
-	
+
 	// No more items ?
 	if (mItems.isEmpty()) {
 		// Done
 		LOG("emitting result");
 		emit result(this);
 		delete this;
-	} else {
-		// First, stat the orig file
-		mState = STATE_STATORIG;
-		mOriginalTime = 0;
-		assert( mNextItem == mItems.current());
-		mCurrentItem = mNextItem;
-		mCurrentURL = mCurrentItem->url();
-		mCurrentURL.cleanPath();
-		// Do direct stat instead of using KIO if the file is local (faster)
-		if( mCurrentURL.isLocalFile()
-			&& !KIO::probably_slow_mounted( mCurrentURL.path())) {
-			KDE_struct_stat buff;
-			if ( KDE_stat( QFile::encodeName(mCurrentURL.path()), &buff ) == 0 )  {
-				mOriginalTime = buff.st_mtime;
-				QTimer::singleShot( 0, this, SLOT( checkThumbnail()));
-			}
+		return;
+	}
+
+	mCurrentItem=mItems.first();
+	mItems.pop_front();
+	Q_ASSERT( !mProcessedState[ thumbnailIndex( mCurrentItem )] );
+	mProcessedState[ thumbnailIndex( mCurrentItem )] = true;
+
+	// First, stat the orig file
+	mState = STATE_STATORIG;
+	mOriginalTime = 0;
+	mCurrentURL = mCurrentItem->url();
+	mCurrentURL.cleanPath();
+
+	// Do direct stat instead of using KIO if the file is local (faster)
+	if( mCurrentURL.isLocalFile()
+		&& !KIO::probably_slow_mounted( mCurrentURL.path())) {
+		KDE_struct_stat buff;
+		if ( KDE_stat( QFile::encodeName(mCurrentURL.path()), &buff ) == 0 )  {
+			mOriginalTime = buff.st_mtime;
+			QTimer::singleShot( 0, this, SLOT( checkThumbnail()));
 		}
-		if( mOriginalTime == 0 ) { // KIO must be used
-			KIO::Job* job = KIO::stat(mCurrentURL,false);
-			LOG( "KIO::stat orig " << mCurrentURL.url() );
-			addSubjob(job);
-		}
-		mItems.remove();
-		mNextItem = mItems.current();
+	}
+	if( mOriginalTime == 0 ) { // KIO must be used
+		KIO::Job* job = KIO::stat(mCurrentURL,false);
+		LOG( "KIO::stat orig " << mCurrentURL.url() );
+		addSubjob(job);
 	}
 }
 
@@ -462,7 +555,7 @@ void ThumbnailLoadJob::slotResult(KIO::Job * job) {
 		KIO::UDSEntry entry = static_cast<KIO::StatJob*>(job)->statResult();
 		KIO::UDSEntry::ConstIterator it= entry.begin();
 		mOriginalTime = 0;
-		for (; it!=entry.end(); it++) {
+		for (; it!=entry.end(); ++it) {
 			if ((*it).m_uds == KIO::UDS_MODIFICATION_TIME) {
 				mOriginalTime = (time_t)((*it).m_long);
 				break;
@@ -475,21 +568,14 @@ void ThumbnailLoadJob::slotResult(KIO::Job * job) {
 	case STATE_DOWNLOADORIG: 
 		if (job->error()) {
 			emitThumbnailLoadingFailed();
-			mState=STATE_DELETETEMP;
-			LOG("Delete temp file " << mTempURL.prettyURL());
-			addSubjob(KIO::file_delete(mTempURL,false));
-			mTempURL = KURL();
+			LOG("Delete temp file " << mTempPath);
+			QFile::remove(mTempPath);
+			mTempPath = QString::null;
+			determineNextIcon();
 		} else {
-			startCreatingThumbnail(mTempURL.path());
+			startCreatingThumbnail(mTempPath);
 		}
 		return;
-
-	case STATE_CREATETHUMB:
-		assert( false ); // thumbnailReady() must handle this
-
-	case STATE_DELETETEMP:
-		determineNextIcon();
-		return;
 	}
 }
 
@@ -501,14 +587,12 @@ void ThumbnailLoadJob::thumbnailReady( const QImage& im ) {
 	} else {
 		emitThumbnailLoadingFailed();
 	}
-	if( !mTempURL.isEmpty()) {
-		mState=STATE_DELETETEMP;
-		LOG("Delete temp file " << mTempURL.prettyURL());
-		addSubjob(KIO::file_delete(mTempURL,false));
-		mTempURL = KURL();
-	} else {
-		determineNextIcon();
+	if( !mTempPath.isEmpty()) {
+		LOG("Delete temp file " << mTempPath);
+		QFile::remove(mTempPath);
+		mTempPath = QString::null;
 	}
+	determineNextIcon();
 }
 
 void ThumbnailLoadJob::checkThumbnail() {
@@ -545,8 +629,10 @@ void ThumbnailLoadJob::checkThumbnail() {
 	} else {
 		mState=STATE_DOWNLOADORIG;
 		KTempFile tmpFile;
-		mTempURL.setPath(tmpFile.name());
-		KIO::Job* job=KIO::file_copy(mCurrentURL,mTempURL,-1,true,false,false);
+		mTempPath=tmpFile.name();
+		KURL url;
+		url.setPath(mTempPath);
+		KIO::Job* job=KIO::file_copy(mCurrentURL, url,-1,true,false,false);
 		LOG("Download remote file " << mCurrentURL.prettyURL());
 		addSubjob(job);
 	}
@@ -554,14 +640,13 @@ void ThumbnailLoadJob::checkThumbnail() {
 
 void ThumbnailLoadJob::startCreatingThumbnail(const QString& pixPath) {
 	LOG("Creating thumbnail from " << pixPath);
-	mThumbnailThread.load( mOriginalURI, mOriginalTime, mCurrentItem->size(), mCurrentItem->mimetype(), pixPath, mThumbnailPath);
-	mState = STATE_CREATETHUMB;
+	mThumbnailThread.load( mOriginalURI, mOriginalTime, mCurrentItem->size(),
+		mCurrentItem->mimetype(), pixPath, mThumbnailPath, mThumbnailSize,
+		sStoreThumbnailsInCache);
 }
 
 
 void ThumbnailLoadJob::emitThumbnailLoaded(const QImage& img) {
-	ThumbnailSize biggest=ThumbnailSize::biggest();
-
 	int biggestDimension=QMAX(img.width(), img.height());
 	bool ok;
 	QSize size;
@@ -589,4 +674,29 @@ void ThumbnailLoadJob::emitThumbnailLoadingFailed() {
 }
 
 
+//---------------------------------------------------------------------------
+//
+// Config
+//
+//---------------------------------------------------------------------------
+void ThumbnailLoadJob::readConfig(KConfig* config, const QString& group) {
+	config->setGroup(group);
+	sStoreThumbnailsInCache=config->readBoolEntry(CONFIG_STORE_THUMBNAILS_IN_CACHE,true);
+}
+
+
+void ThumbnailLoadJob::writeConfig(KConfig* config, const QString& group) {
+	config->setGroup(group);
+	config->writeEntry(CONFIG_STORE_THUMBNAILS_IN_CACHE,sStoreThumbnailsInCache);
+}
+
+
+void ThumbnailLoadJob::setStoreThumbnailsInCache(bool store) {
+	sStoreThumbnailsInCache=store;
+}
+
+
+bool ThumbnailLoadJob::storeThumbnailsInCache() {
+	return sStoreThumbnailsInCache;
+}
 
